# (C) Copyright 1996- ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.

# Note: This script is intended only as example usage of thermofeel library.
#       It is designed to be used with ECMWF forecast data.
#       The function ifs_step_intervals() is used to calculate the time interval based on the forecast step.
#       This is particular to the IFS model and ECMWF's NWP operational system.

import argparse
import csv
import sys

import dateutil.parser
import numpy as np

import thermofeel as thermofeel

#######################################################################################################################

# INPUTS
# BAR_dif.tab   DIF [W/m**2]  -- ssrd = ssrdir + ssrdiff --> ssrdiff = ssrd - fdir
# BAR_dir.tab   DIR [W/m**2]  --
# BAR_lwd.tab   LWD [W/m**2]  -- strd   (thermal radiadtion downwards)
# BAR_lwu.tab   LWU [W/m**2]  -- stru = strd - strr   thermal radiadtion upwards
# BAR_rh.tab    RH  [%]       -- use to compute td with calculate_dew_point_from_relative_humidity(rh, t2m)
# BAR_swd.tab   SWD [W/m**2]  -- ssrd
# BAR_swu.tab   SWU [W/m**2]  -- ssr = swd - swu --> swu = swd - ssr --> swu = ssrd - ssr
# BAR_t2m.tab   T2 [°C]       -- t2m

# ssrd = swd
# ssr = ssrd - swu
# dsrp =
# strd = lwd
# fdir =
# strr = lwd - lwu
# cossza


# Five stations with wind data, IDS: BAR  IZA  NYA  PAY   TAT

# files with names <STATIONID>_{dif,dir,lwd,lwu,rh,swd,swu,t2m,wind}.tab

# OUTPUTS, CSV file with each station
# cossza
# mrt
# wbgt

#######################################################################################################################


def command_line_options():
    parser = argparse.ArgumentParser()

    parser.add_argument("output", help="output filename to dump results in CSV format")
    parser.add_argument("--basepath", help="basename for input files")
    parser.add_argument(
        "--stationid", help="station 3 char identifier (e.g. TAT for Tateno) "
    )

    parser.add_argument("--max", help="max number of items", type=int, default=0)

    args = parser.parse_args()

    return args


def read_part_dataset(obsdata, varname, filepath, nmax=0):
    with open(filepath, newline="") as f:
        reader = csv.reader(f, delimiter="\t")
        next(reader)  # skip first line

        nitems = 0
        for record in reader:
            x = record[0]
            y = record[1]
            if y.startswith("ø"):
                y = float(y[1:])

            # print(x, y)
            if x in obsdata:
                obsdata[x].update([(varname, y)])
            else:
                obsdata[x] = dict([(varname, y)])

            nitems = nitems + 1

            if nmax and nitems >= nmax:
                break

        print(f"{filepath}: {nitems} record(s)")


def station_location(id):
    # IZA () * Latitude:  * Longitude:
    # NYA () * Latitude: 78.922700 * Longitude:
    #
    return {
        "PAY": {
            "location": "Payerne",
            "lat": 46.812300,
            "lon": 6.942200,
        },  # 'url': 'https://doi.pangaea.de/10.1594/PANGAEA.671565' },
        "TAT": {
            "location": "Tateno",
            "lat": 36.058100,
            "lon": 140.125800,
        },  # 'url': 'https://doi.pangaea.de/10.1594/PANGAEA.673090' },
        "BAR": {
            "location": "Barrow",
            "lat": 71.323000,
            "lon": -156.607000,
        },  # 'url': 'https://doi.pangaea.de/10.1594/PANGAEA.789103' },
        "IZA": {
            "location": "   ",
            "lat": 28.309350,
            "lon": -16.499260,
        },  # 'url': 'https://doi.pangaea.de/10.1594/PANGAEA.871032' },
        "NYA": {
            "location": "   ",
            "lat": 78.922700,
            "lon": 11.927300,
        },  # 'url': 'https://doi.pangaea.de/10.1594/PANGAEA.670762' },
    }[id]


def main():
    args = command_line_options()

    print(f"Thermofeel version: {thermofeel.__version__}")
    # print(f"Python version: {sys.version}")
    # print(f"Numpy version: {np.version.version}")
    # np.show_config()

    station = station_location(args.stationid)
    location = station["location"]
    lat = station["lat"]
    lon = station["lon"]

    print(f"stationid {args.stationid} in {location} @ lat {lat} lon {lon}")

    obsdata = {}

    read_part_dataset(
        obsdata, "rh", args.basepath + args.stationid + "_rh.tab", args.max
    )
    read_part_dataset(
        obsdata, "t2m", args.basepath + args.stationid + "_t2m.tab", args.max
    )
    read_part_dataset(
        obsdata, "wind", args.basepath + args.stationid + "_wind.tab", args.max
    )
    read_part_dataset(
        obsdata, "dif", args.basepath + args.stationid + "_dif.tab", args.max
    )
    read_part_dataset(
        obsdata, "dir", args.basepath + args.stationid + "_dir.tab", args.max
    )
    read_part_dataset(
        obsdata, "lwd", args.basepath + args.stationid + "_lwd.tab", args.max
    )
    read_part_dataset(
        obsdata, "lwu", args.basepath + args.stationid + "_lwu.tab", args.max
    )
    read_part_dataset(
        obsdata, "swd", args.basepath + args.stationid + "_swd.tab", args.max
    )
    read_part_dataset(
        obsdata, "swu", args.basepath + args.stationid + "_swu.tab", args.max
    )

    print(f"total entries {len(obsdata.keys())}")

    completes = []
    for k in obsdata.keys():
        entry = obsdata[k]

        if (
            "rh" in entry
            and "t2m" in entry
            and "wind" in entry
            and "dif" in entry
            and "dir" in entry
            and "lwd" in entry
            and "lwu" in entry
            and "swd" in entry
            and "swu" in entry
        ):
            entry.update(station)
            entry.update({"stationid": args.stationid})
            entry.update({"datetime": k})

            dt = dateutil.parser.isoparse(k)
            # print(k,  entry)
            # print(dt)

            cossza = thermofeel.calculate_cos_solar_zenith_angle(
                lat=lat, lon=lon, y=dt.year, m=dt.month, d=dt.day, h=dt.hour
            )

            rh = float(entry["rh"])  # percent
            t2m = thermofeel.celsius_to_kelvin(float(entry["t2m"]))  # K
            va = float(entry["wind"])  # m/s
            dir = float(entry["dir"])  # m/s
            swd = float(entry["swd"])  # W/m**2
            swu = float(entry["swu"])  # W/m**2
            lwd = float(entry["lwd"])  # W/m**2
            lwu = float(entry["lwu"])  # W/m**2

            rh = np.array([rh])
            t2m = np.array([t2m])
            va = np.array([va])
            dir = np.array([dir])
            swd = np.array([swd])
            swu = np.array([swu])
            lwd = np.array([lwd])
            lwu = np.array([lwu])

            ssrd = swd
            ssr = ssrd - swu
            dsrp = dir
            strd = lwd
            strr = lwd - lwu
            fdir = dsrp * cossza

            td = thermofeel.calculate_dew_point_from_relative_humidity(rh, t2m)
            mrt = thermofeel.calculate_mean_radiant_temperature(
                ssrd, ssr, dsrp, strd, fdir, strr, cossza
            )
            wbgt = thermofeel.calculate_wbgt(t2m, mrt, va, td)

            # print(f"cossza={cossza} mrt {mrt} mrt_C {thermofeel.kelvin_to_celsius(mrt)} wbgt {wbgt}")

            entry.update(
                {"cossza": cossza, "td": td[0], "mrt": mrt[0], "wbgt": wbgt[0]}
            )

            completes.append(entry)

    print(f"complete entries {len(completes)}")

    if len(completes):
        try:
            outfile = args.output
            columns = [
                "stationid",
                "location",
                "lat",
                "lon",
                "datetime",
                "rh",
                "t2m",
                "wind",
                "dif",
                "dir",
                "lwd",
                "lwu",
                "swd",
                "swu",
                "cossza",
                "td",
                "mrt",
                "wbgt",
            ]
            print(f"creating output CSV {outfile} with columns {columns}")
            with open(outfile, "w") as f:
                writer = csv.DictWriter(f, fieldnames=columns)
                writer.writeheader()
                for e in completes:
                    writer.writerow(e)
        except IOError:
            print("I/O error")

    else:
        print("No complete set of observations to output")


if __name__ == "__main__":
    sys.exit(main())



# (C) Copyright 1996- ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.

"""
thermofeel is a library to calculate human thermal comfort indexes.

  Currently calculates the thermal indexes:
  * Universal Thermal Climate Index
  * Apparent Temperature
  * Heat Index Adjusted
  * Heat Index Simplified
  * Humidex
  * Normal Effective Temperature
  * Wet Bulb Globe Temperature
  * Wet Bulb Globe Temperature Simple
  * Wind Chill

  In support of the above indexes, it also calculates:
  * Globe Temperature
  * Mean Radiant Temperature
  * Mean Radiant Temperature from Globe Temperature
  * Relative Humidity Percentage
  * Saturation vapour pressure
  * Wet Bulb Temperature

  To calculate the cos of the solar zenith angle, we suggest to use the
  earthkit-meteo library (github.com:ecmwf/earthkit-meteo.git)
"""

import math

import numpy as np

from .helpers import (
    celsius_to_kelvin,
    fahrenheit_to_kelvin,
    kelvin_to_celsius,
    kelvin_to_fahrenheit,
)

to_radians = math.pi / 180.0


def calculate_relative_humidity_percent(t2_k, td_k):
    """
    Relative Humidity in percent
        :param t2_k: (float array) 2m temperature [K]
        :param td_k: (float array) dew point temperature [K]
        returns relative humidity [%]
    """

    t2_c = kelvin_to_celsius(t2_k)
    td_c = kelvin_to_celsius(td_k)
    # saturated vapour pressure
    es = 6.11 * 10.0 ** (7.5 * t2_c / (237.3 + t2_c))
    # vapour pressure
    e = 6.11 * 10.0 ** (7.5 * td_c / (237.3 + td_c))
    rh = (e / es) * 100
    return rh


def calculate_saturation_vapour_pressure(t2_k):
    """
    Saturation vapour pressure over water
        :param t2_k: (float array) 2m temperature [K]
        returns saturation vapor pressure over water in the pure phase [hPa] == [mBar]
    Reference: Hardy (1998)
    https://www.decatur.de/javascript/dew/resources/its90formulas.pdf
    """

    g = [
        -2.8365744e3,
        -6.028076559e3,
        1.954263612e1,
        -2.737830188e-2,
        1.6261698e-5,
        7.0229056e-10,
        -1.8680009e-13,
        2.7150305,
    ]
    ess = g[7] * np.log(t2_k)
    for i in range(7):
        ess = ess + g[i] * np.power(t2_k, (i - 2))

    ess = np.exp(ess) * 0.01  # hPa

    return ess


def calculate_saturation_vapour_pressure_multiphase(t2_k, phase):
    """
    Saturation vapour pressure over liquid water and ice
        :param t2_k: (float array) 2m temperature [K]
        :param phase: 0 over liquid water and 1 over ice
        returns pressure of water vapor over a surface of liquid water or ice [hPa] == [mBar]
    Reference: ECMWF IFS Documentation CY45R1 - Part IV : Physical processes (2018) pp. 116
    https://doi.org/10.21957/4whwo8jw0
    https://metview.readthedocs.io/en/latest/api/functions/saturation_vapour_pressure.html
    """
    T0 = 273.16  # triple point of water 273.16 K (0.01 °C) at 611.73 Pa
    es = np.zeros_like(t2_k)
    y = (t2_k - T0) / (t2_k - 32.19)  # over liquid water
    es[phase == 0] = 6.1121 * np.exp(17.502 * y[phase == 0])
    y = (t2_k - T0) / (t2_k + 0.7)  # over ice
    es[phase == 1] = 6.1121 * np.exp(22.587 * y[phase == 1])

    return es


def calculate_nonsaturation_vapour_pressure(t2_k, rh):
    """
    Non saturated vapour pressure
        :param t2_k: (float array) 2m temperature [K]
        :param rh: (float array) relative humidity percentage [%]
        returns non saturated vapor pressure [hPa] == [mBar]
    Reference: Bureau of Meteorology (2010)
    http://www.bom.gov.au/info/thermal_stress/#approximation
    """

    t2_c = kelvin_to_celsius(t2_k)
    ens = rh / 100 * 6.105 * np.exp(17.27 * t2_c / (237.7 + t2_c))

    return ens


def scale_windspeed(va, h):
    """
    Scaling wind speed from 10 metres to height h
        :param va: (float array) 10m wind speed [m/s]
        :param h: (float array) height at which wind speed needs to be scaled [m]
        returns wind speed at height h
    Reference: Bröde et al. (2012)
    https://doi.org/10.1007/s00484-011-0454-1
    """
    c = 1 / np.log10(10 / 0.01)  #
    c = 0.333333333333
    vh = va * np.log10(h / 0.01) * c

    return vh


def approximate_dsrp(fdir, cossza, threshold=0.1):
    """
    Helper function to approximate dsrp from fdir and cossza
    Note that the function introduces large errors as cossza approaches zero.
    Only use if dsrp is not available in your dataset.
    To compute cossza consider using earhkit-meteo.solar.calculate_cos_solar_zenith_angle
        :param fdir: (float array) total sky direct solar radiation at surface [W m-2]
        :param cossza: (float array) cosine of solar zenith angle [dimentionless]
        returns direct radiation from the Sun [W m-2]
    """
    # filter statement for solar zenith angle to avoid division by zero.
    csza_filter1 = np.where((cossza > threshold))
    dsrp = np.copy(fdir)  # dsrp = fdir for cossza <= 0.01, equals to fdir
    dsrp[csza_filter1] = dsrp[csza_filter1] / cossza[csza_filter1]
    return dsrp


def calculate_dew_point_from_relative_humidity(rh, t2_k):
    """
    Dew point temperature at 2m from relative humidity in percent
        :param rh: (float array) relative humidity [%]
        :param t2_k: (float array) 2m temperature [K]
        returns dew point temperature [K]
    Reference: Alduchov and Eskridge (1996)
    https://doi.org/10.1175/1520-0450(1996)035<0601:IMFAOS>2.0.CO;2
    """
    t2_c = kelvin_to_celsius(t2_k)
    td_c = (
        243.04
        * (np.log(rh / 100) + ((17.625 * t2_c) / (243.04 + t2_c)))
        / (17.625 - np.log(rh / 100) - ((17.625 * t2_c) / (243.04 + t2_c)))
    )
    td_k = celsius_to_kelvin(td_c)
    return td_k


def calculate_mean_radiant_temperature(ssrd, ssr, dsrp, strd, fdir, strr, cossza):
    """
    MRT - Mean Radiant Temperature
    To compute cossza consider using earhkit-meteo.solar.calculate_cos_solar_zenith_angle
        :param ssrd: (float array) surface solar radiation downwards [W m-2]
        :param ssr: (float array) surface net solar radiation [W m-2]
        :param dsrp: (float array) direct solar radiation [W m-2]
        :param strd: (float array) surface thermal radiation downwards [W m-2]
        :param fdir: (float array) total sky direct solar radiation at surface [W m-2]
        :param strr: (float array) surface net thermal radiation [W m-2]
        :param cossza: (float array) cosine of solar zenith angle [dimentionless]
        returns mean radiant temperature [K]
    Reference: Di Napoli et al. (2020)
    https://link.springer.com/article/10.1007/s00484-020-01900-5
    """

    dsw = ssrd - fdir
    rsw = ssrd - ssr
    lur = strd - strr
    # Istar = dsrp

    # calculate fp projected factor area

    gamma = np.arcsin(cossza) * 180 / np.pi
    fp = 0.308 * np.cos(to_radians * gamma * (0.998 - gamma * gamma / 50000))

    # calculate mean radiant temperature
    mrt = np.power(
        (
            (1 / 0.0000000567)
            * (
                0.5 * strd
                + 0.5 * lur
                + (0.7 / 0.97) * (0.5 * dsw + 0.5 * rsw + fp * dsrp)
            )
        ),
        0.25,
    )

    return mrt


def calculate_utci_polynomial(t2m, mrt, va, wvp):
    """
    Helper function to calculate the UTCI polynomial approximation
        :param t2_k: (float array) is 2m temperature [K]
        :param mrt: (float array) is mean radiant temperature [K]
        :param va: (float array) is wind speed at 10 meters [m/s]
        :param wvp: (float array) is water vapour pressure [kPa]
    returns UTCI [K]
    Reference: Brode et al. (2012)
    https://doi.org/10.1007/s00484-011-0454-1
    """
    e_mrt = np.subtract(mrt, t2m)

    t2m2 = t2m * t2m
    t2m3 = t2m2 * t2m
    t2m4 = t2m3 * t2m
    t2m5 = t2m4 * t2m
    t2m6 = t2m5 * t2m

    va2 = va * va
    va3 = va2 * va
    va4 = va3 * va
    va5 = va4 * va
    va6 = va5 * va

    e_mrt2 = e_mrt * e_mrt
    e_mrt3 = e_mrt2 * e_mrt
    e_mrt4 = e_mrt3 * e_mrt
    e_mrt5 = e_mrt4 * e_mrt
    e_mrt6 = e_mrt5 * e_mrt

    wvp2 = wvp * wvp
    wvp3 = wvp2 * wvp
    wvp4 = wvp3 * wvp
    wvp5 = wvp4 * wvp
    wvp6 = wvp5 * wvp

    varh2 = va * wvp2
    va2_rh = va2 * wvp
    va2_e_mrt = va2 * e_mrt
    e_mrt_rh = e_mrt * wvp
    e_mrt_rh2 = e_mrt * wvp2
    e_mrt2_rh = e_mrt2 * wvp
    e_mrt2_rh2 = e_mrt2 * wvp2
    e_mrt_rh3 = e_mrt * wvp3
    va_e_mrt = va * e_mrt
    va_e_mrt2 = va * e_mrt2
    va_rh = va * wvp
    t2m_va = t2m * va
    e_mrt3_rh = e_mrt3 * wvp
    e_mrt4_rh = e_mrt4 * wvp

    utci = (
        t2m
        + 6.07562052e-01
        + -2.27712343e-02 * t2m
        + 8.06470249e-04 * t2m2
        + -1.54271372e-04 * t2m3
        + -3.24651735e-06 * t2m4
        + 7.32602852e-08 * t2m5
        + 1.35959073e-09 * t2m6
        + -2.25836520e00 * va
        + 8.80326035e-02 * t2m * va
        + 2.16844454e-03 * t2m2 * va
        + -1.53347087e-05 * t2m3 * va
        + -5.72983704e-07 * t2m4 * va
        + -2.55090145e-09 * t2m5 * va
        + -7.51269505e-01 * va2
        + -4.08350271e-03 * t2m * va2
        + -5.21670675e-05 * t2m2 * va2
        + 1.94544667e-06 * t2m3 * va2
        + 1.14099531e-08 * t2m4 * va2
        + 1.58137256e-01 * va3
        + -6.57263143e-05 * t2m * va3
        + 2.22697524e-07 * t2m2 * va3
        + -4.16117031e-08 * t2m3 * va3
        + -1.27762753e-02 * va4
        + 9.66891875e-06 * t2m * va4
        + 2.52785852e-09 * t2m2 * va4
        + 4.56306672e-04 * va5
        + -1.74202546e-07 * t2m * va5
        + -5.91491269e-06 * va6
        + 3.98374029e-01 * e_mrt
        + 1.83945314e-04 * t2m * e_mrt
        + -1.73754510e-04 * t2m2 * e_mrt
        + -7.60781159e-07 * t2m3 * e_mrt
        + 3.77830287e-08 * t2m4 * e_mrt
        + 5.43079673e-10 * t2m5 * e_mrt
        + -2.00518269e-02 * va_e_mrt
        + 8.92859837e-04 * t2m * va_e_mrt
        + 3.45433048e-06 * t2m2 * va_e_mrt
        + -3.77925774e-07 * t2m3 * va_e_mrt
        + -1.69699377e-09 * t2m4 * va_e_mrt
        + 1.69992415e-04 * va2_e_mrt
        + -4.99204314e-05 * t2m * va2_e_mrt
        + 2.47417178e-07 * t2m2 * va2_e_mrt
        + 1.07596466e-08 * t2m3 * va2_e_mrt
        + 8.49242932e-05 * va3 * e_mrt
        + 1.35191328e-06 * t2m * va3 * e_mrt
        + -6.21531254e-09 * t2m2 * va3 * e_mrt
        + -4.99410301e-06 * va4 * e_mrt
        + -1.89489258e-08 * t2m * va4 * e_mrt
        + 8.15300114e-08 * va5 * e_mrt
        + 7.55043090e-04 * e_mrt2
        + -5.65095215e-05 * t2m * e_mrt2
        + -4.52166564e-07 * t2m2 * e_mrt2
        + 2.46688878e-08 * t2m3 * e_mrt2
        + 2.42674348e-10 * t2m4 * e_mrt2
        + 1.54547250e-04 * va_e_mrt2
        + 5.24110970e-06 * t2m * va_e_mrt2
        + -8.75874982e-08 * t2m2 * va_e_mrt2
        + -1.50743064e-09 * t2m3 * va_e_mrt2
        + -1.56236307e-05 * va2 * e_mrt2
        + -1.33895614e-07 * t2m * va2 * e_mrt2
        + 2.49709824e-09 * t2m2 * va2 * e_mrt2
        + 6.51711721e-07 * va3 * e_mrt2
        + 1.94960053e-09 * t2m * va3 * e_mrt2
        + -1.00361113e-08 * va4 * e_mrt2
        + -1.21206673e-05 * e_mrt3
        + -2.18203660e-07 * t2m * e_mrt3
        + 7.51269482e-09 * t2m2 * e_mrt3
        + 9.79063848e-11 * t2m3 * e_mrt3
        + 1.25006734e-06 * va * e_mrt3
        + -1.81584736e-09 * t2m_va * e_mrt3
        + -3.52197671e-10 * t2m2 * va * e_mrt3
        + -3.36514630e-08 * va2 * e_mrt3
        + 1.35908359e-10 * t2m * va2 * e_mrt3
        + 4.17032620e-10 * va3 * e_mrt3
        + -1.30369025e-09 * e_mrt4
        + 4.13908461e-10 * t2m * e_mrt4
        + 9.22652254e-12 * t2m2 * e_mrt4
        + -5.08220384e-09 * va * e_mrt4
        + -2.24730961e-11 * t2m_va * e_mrt4
        + 1.17139133e-10 * va2 * e_mrt4
        + 6.62154879e-10 * e_mrt5
        + 4.03863260e-13 * t2m * e_mrt5
        + 1.95087203e-12 * va * e_mrt5
        + -4.73602469e-12 * e_mrt6
        + 5.12733497e00 * wvp
        + -3.12788561e-01 * t2m * wvp
        + -1.96701861e-02 * t2m2 * wvp
        + 9.99690870e-04 * t2m3 * wvp
        + 9.51738512e-06 * t2m4 * wvp
        + -4.66426341e-07 * t2m5 * wvp
        + 5.48050612e-01 * va_rh
        + -3.30552823e-03 * t2m * va_rh
        + -1.64119440e-03 * t2m2 * va_rh
        + -5.16670694e-06 * t2m3 * va_rh
        + 9.52692432e-07 * t2m4 * va_rh
        + -4.29223622e-02 * va2_rh
        + 5.00845667e-03 * t2m * va2_rh
        + 1.00601257e-06 * t2m2 * va2_rh
        + -1.81748644e-06 * t2m3 * va2_rh
        + -1.25813502e-03 * va3 * wvp
        + -1.79330391e-04 * t2m * va3 * wvp
        + 2.34994441e-06 * t2m2 * va3 * wvp
        + 1.29735808e-04 * va4 * wvp
        + 1.29064870e-06 * t2m * va4 * wvp
        + -2.28558686e-06 * va5 * wvp
        + -3.69476348e-02 * e_mrt_rh
        + 1.62325322e-03 * t2m * e_mrt_rh
        + -3.14279680e-05 * t2m2 * e_mrt_rh
        + 2.59835559e-06 * t2m3 * e_mrt_rh
        + -4.77136523e-08 * t2m4 * e_mrt_rh
        + 8.64203390e-03 * va * e_mrt_rh
        + -6.87405181e-04 * t2m_va * e_mrt_rh
        + -9.13863872e-06 * t2m2 * va * e_mrt_rh
        + 5.15916806e-07 * t2m3 * va * e_mrt_rh
        + -3.59217476e-05 * va2 * e_mrt_rh
        + 3.28696511e-05 * t2m * va2 * e_mrt_rh
        + -7.10542454e-07 * t2m2 * va2 * e_mrt_rh
        + -1.24382300e-05 * va3 * e_mrt_rh
        + -7.38584400e-09 * t2m * va3 * e_mrt_rh
        + 2.20609296e-07 * va4 * e_mrt_rh
        + -7.32469180e-04 * e_mrt2_rh
        + -1.87381964e-05 * t2m * e_mrt2_rh
        + 4.80925239e-06 * t2m2 * e_mrt2_rh
        + -8.75492040e-08 * t2m3 * e_mrt2_rh
        + 2.77862930e-05 * va * e_mrt2_rh
        + -5.06004592e-06 * t2m_va * e_mrt2_rh
        + 1.14325367e-07 * t2m2 * va * e_mrt2_rh
        + 2.53016723e-06 * va2 * e_mrt2_rh
        + -1.72857035e-08 * t2m * va2 * e_mrt2_rh
        + -3.95079398e-08 * va3 * e_mrt2_rh
        + -3.59413173e-07 * e_mrt3_rh
        + 7.04388046e-07 * t2m * e_mrt3_rh
        + -1.89309167e-08 * t2m2 * e_mrt3_rh
        + -4.79768731e-07 * va * e_mrt3_rh
        + 7.96079978e-09 * t2m_va * e_mrt3_rh
        + 1.62897058e-09 * va2 * e_mrt3_rh
        + 3.94367674e-08 * e_mrt4_rh
        + -1.18566247e-09 * t2m * e_mrt4_rh
        + 3.34678041e-10 * va * e_mrt4_rh
        + -1.15606447e-10 * e_mrt5 * wvp
        + -2.80626406e00 * wvp2
        + 5.48712484e-01 * t2m * wvp2
        + -3.99428410e-03 * t2m2 * wvp2
        + -9.54009191e-04 * t2m3 * wvp2
        + 1.93090978e-05 * t2m4 * wvp2
        + -3.08806365e-01 * varh2
        + 1.16952364e-02 * t2m * varh2
        + 4.95271903e-04 * t2m2 * varh2
        + -1.90710882e-05 * t2m3 * varh2
        + 2.10787756e-03 * va2 * wvp2
        + -6.98445738e-04 * t2m * va2 * wvp2
        + 2.30109073e-05 * t2m2 * va2 * wvp2
        + 4.17856590e-04 * va3 * wvp2
        + -1.27043871e-05 * t2m * va3 * wvp2
        + -3.04620472e-06 * va4 * wvp2
        + 5.14507424e-02 * e_mrt_rh2
        + -4.32510997e-03 * t2m * e_mrt_rh2
        + 8.99281156e-05 * t2m2 * e_mrt_rh2
        + -7.14663943e-07 * t2m3 * e_mrt_rh2
        + -2.66016305e-04 * va * e_mrt_rh2
        + 2.63789586e-04 * t2m_va * e_mrt_rh2
        + -7.01199003e-06 * t2m2 * va * e_mrt_rh2
        + -1.06823306e-04 * va2 * e_mrt_rh2
        + 3.61341136e-06 * t2m * va2 * e_mrt_rh2
        + 2.29748967e-07 * va3 * e_mrt_rh2
        + 3.04788893e-04 * e_mrt2_rh2
        + -6.42070836e-05 * t2m * e_mrt2_rh2
        + 1.16257971e-06 * t2m2 * e_mrt2_rh2
        + 7.68023384e-06 * va * e_mrt2_rh2
        + -5.47446896e-07 * t2m_va * e_mrt2_rh2
        + -3.59937910e-08 * va2 * e_mrt2_rh2
        + -4.36497725e-06 * e_mrt3 * wvp2
        + 1.68737969e-07 * t2m * e_mrt3 * wvp2
        + 2.67489271e-08 * va * e_mrt3 * wvp2
        + 3.23926897e-09 * e_mrt4 * wvp2
        + -3.53874123e-02 * wvp3
        + -2.21201190e-01 * t2m * wvp3
        + 1.55126038e-02 * t2m2 * wvp3
        + -2.63917279e-04 * t2m3 * wvp3
        + 4.53433455e-02 * va * wvp3
        + -4.32943862e-03 * t2m_va * wvp3
        + 1.45389826e-04 * t2m2 * va * wvp3
        + 2.17508610e-04 * va2 * wvp3
        + -6.66724702e-05 * t2m * va2 * wvp3
        + 3.33217140e-05 * va3 * wvp3
        + -2.26921615e-03 * e_mrt_rh3
        + 3.80261982e-04 * t2m * e_mrt_rh3
        + -5.45314314e-09 * t2m2 * e_mrt_rh3
        + -7.96355448e-04 * va * e_mrt_rh3
        + 2.53458034e-05 * t2m_va * e_mrt_rh3
        + -6.31223658e-06 * va2 * e_mrt_rh3
        + 3.02122035e-04 * e_mrt2 * wvp3
        + -4.77403547e-06 * t2m * e_mrt2 * wvp3
        + 1.73825715e-06 * va * e_mrt2 * wvp3
        + -4.09087898e-07 * e_mrt3 * wvp3
        + 6.14155345e-01 * wvp4
        + -6.16755931e-02 * t2m * wvp4
        + 1.33374846e-03 * t2m2 * wvp4
        + 3.55375387e-03 * va * wvp4
        + -5.13027851e-04 * t2m_va * wvp4
        + 1.02449757e-04 * va2 * wvp4
        + -1.48526421e-03 * e_mrt * wvp4
        + -4.11469183e-05 * t2m * e_mrt * wvp4
        + -6.80434415e-06 * va * e_mrt * wvp4
        + -9.77675906e-06 * e_mrt2 * wvp4
        + 8.82773108e-02 * wvp5
        + -3.01859306e-03 * t2m * wvp5
        + 1.04452989e-03 * va * wvp5
        + 2.47090539e-04 * e_mrt * wvp5
        + 1.48348065e-03 * wvp6
    )

    return utci


def calculate_utci(t2_k, va, mrt, td_k=None, ehPa=None):
    """
    UTCI - Universal Thermal Climate Index
        :param t2_k: (float array) is 2m temperature [K]
        :param va: (float array) is wind speed at 10 meters [m/s]
        :param mrt: (float array) is mean radiant temperature [K]
        :param td_k: (float array) is 2m dew point temperature [K]
        :param ehPa: (float array) is water vapour pressure [hPa]
    returns UTCI [K]
    Reference: Brode et al. (2012)
    https://doi.org/10.1007/s00484-011-0454-1
    """

    if ehPa is not None:
        wvp = ehPa / 10.0  # water vapour pressure in kPa
    else:
        if td_k is not None:
            rh_pc = calculate_relative_humidity_percent(t2_k, td_k)
            ehPa = calculate_saturation_vapour_pressure(t2_k) * rh_pc / 100.0
            wvp = ehPa / 10.0  # water vapour pressure in kPa
        else:
            raise ValueError("Missing input ehPa or td_k")

    t2_c = kelvin_to_celsius(t2_k)  # polynomial approx. is in Celsius
    mrt_c = kelvin_to_celsius(mrt)  # polynomial approx. is in Celsius

    utci = calculate_utci_polynomial(t2_c, mrt_c, va, wvp)
    utci_k = celsius_to_kelvin(utci)

    return utci_k


def calculate_wbgt_simple(t2_k, rh):
    """
    WBGT - Wet Bulb Globe Temperature computed by a the simpler algorithm
        :param t2_k: (float array) 2m temperature [K]
        :param rh: (float array) relative humidity percentage [%]
        returns Wet Bulb Globe Temperature [K]
    Reference: ACSM (1984)
    https://doi.org/10.1080/00913847.1984.11701899
    See also: http://www.bom.gov.au/info/thermal_stress/#approximation
    https://www.jstage.jst.go.jp/article/indhealth/50/4/50_MS1352/_pdf
    """
    t2_c = kelvin_to_celsius(t2_k)
    e = calculate_nonsaturation_vapour_pressure(t2_k, rh)
    wbgt = 0.567 * t2_c + 0.393 * e + 3.94
    wbgt_k = celsius_to_kelvin(wbgt)

    return wbgt_k


def calculate_wbt(t2_k, rh):
    """
    Wet Bulb Temperature
        :param t2_k: (float array) 2m temperature [K]
        :param rh: (float array) relative humidity percentage [%]
        returns wet bulb temperature [K]
    Reference: Stull (2011)
    https://doi.org/10.1175/JAMC-D-11-0143.1
    """
    t2_c = kelvin_to_celsius(t2_k)
    tw = (
        t2_c * np.arctan(0.151977 * np.sqrt(rh + 8.313659))
        + np.arctan(t2_c + rh)
        - np.arctan(rh - 1.676331)
        + 0.00391838 * (rh) ** (3 / 2) * np.arctan(0.023101 * rh)
        - 4.686035
    )
    tw_k = celsius_to_kelvin(tw)

    return tw_k


def calculate_bgt(t2_k, mrt, va):
    """
    Globe temperature
        :param t2_k: (float array) 2m temperature [K]
        :param mrt: (float array) mean radiant temperature [K]
        :param va: (float array) wind speed at 10 meters [m/s]
        returns globe temperature [K]
    Reference: Guo et al. 2018
    https://doi.org/10.1016/j.enbuild.2018.08.029
    """
    v = scale_windspeed(
        va, 1.1
    )  # formula requires wind speed at 1.1m (i.e., at the level of the globe)

    # a = 1
    d = (1.1e8 * v**0.6) / (0.95 * 0.15**0.4)
    e = -(mrt**4) - d * t2_k

    q = 12 * e
    s = 27 * (d**2)
    delta = ((s + np.sqrt(s**2 - 4 * (q**3))) / 2) ** (1 / 3)
    Q = 0.5 * np.sqrt((1 / 3) * (delta + q / delta))

    bgt = -Q + 0.5 * np.sqrt(-4 * (Q**2) + d / Q)

    # f = (1.1e8 * va**0.6) / (0.95 * 0.15**0.4)
    # a = f / 2
    # b = -f * t2_k - mrt**4
    # rt1 = 3 ** (1 / 3)
    # rt2 = np.sqrt(3) * np.sqrt(27 * a**4 - 16 * b**3) + 9 * a**2
    # rt3 = 2 * 2 ** (2 / 3) * b
    # a = a.clip(min=0)
    # bgt = -1 / 2 * np.sqrt(
    #     rt3 / (rt1 * rt2 ** (1 / 3)) + (2 ** (1 / 3) * rt2 ** (1 / 3)) / 3 ** (2 / 3)
    # ) + 1 / 2 * np.sqrt(
    #     (4 * a)
    #     / np.sqrt(
    #         rt3 / (rt1 * rt2 ** (1 / 3))
    #         + (2 ** (1 / 3) * rt2 ** (1 / 3)) / 3 ** (2 / 3)
    #     )
    #     - (2 ** (1 / 3) * rt2 ** (1 / 3)) / 3 ** (2 / 3)
    #     - rt3 / (rt1 * rt2 ** (1 / 3))
    # )

    return bgt


def calculate_wbgt(t2_k, mrt, va, td_k):
    """
    WBGT - Wet Bulb Globe Temperature
        :param t2_k: (float array) 2m temperature [K]
        :param mrt: (float array) mean radiant temperature [K]
        :param va: (float array) wind speed at 10 meters [m/s]
        :param td_k: (float array) dew point temperature [K]
        returns wet bulb globe temperature [K]
    Reference: Stull (2011)
    https://doi.org/10.1175/JAMC-D-11-0143.1
    See also: http://www.bom.gov.au/info/thermal_stress/
    """

    bgt_k = calculate_bgt(t2_k, mrt, va)
    bgt_c = kelvin_to_celsius(bgt_k)

    rh = calculate_relative_humidity_percent(t2_k, td_k)
    t2_c = kelvin_to_celsius(t2_k)
    tw_k = calculate_wbt(t2_k, rh)
    tw_c = kelvin_to_celsius(tw_k)

    wbgt = 0.7 * tw_c + 0.2 * bgt_c + 0.1 * t2_c
    wbgt_k = celsius_to_kelvin(wbgt)

    return wbgt_k


def calculate_mrt_from_bgt(t2_k, bgt_k, va):
    """
    Mean radiant temperature from globe temperature
        :param t2_k: (float array) 2m temperature [K]
        :param bgt_k: (float array) globe temperature [K]
        :param va: (float array) wind speed at 10 meters [m/s]
        returns mean radiant temperature [K]
    Reference: Brimicombe et al. (2023)
    https://doi.org/10.1029/2022GH000701
    """
    v = scale_windspeed(
        va, 1.1
    )  # formula requires wind speed at 1.1m (i.e., at the level of the globe)
    f = (1.1e8 * v**0.6) / (0.95 * 0.15**0.4)
    bgt4 = bgt_k**4
    mrtc = bgt4 + f * (bgt_k - t2_k)
    mrtc2 = np.sqrt(np.sqrt(mrtc))

    return mrtc2


def calculate_humidex(t2_k, td_k):
    """
        Humidex
        :param t2_k: (float array) 2m temperature [K]
        :param td_k: (float array) dew point temperature [K]
        returns humidex [K]
    Reference: Blazejczyk et al. (2012)
    https://doi.org/10.1007/s00484-011-0453-2
    """
    vp = 6.11 * np.exp(5417.7530 * ((1 / 273.16) - (1 / td_k)))  # vapour pressure [hPa]
    h = 0.5555 * (vp - 10.0)
    humidex = t2_k + h

    return humidex


def calculate_normal_effective_temperature(t2_k, va, rh):
    """
    NET - Normal Effective Temperature
        :param t2_k: (float array) 2m temperature [K]
        :param va: (float array) wind speed at 10 meters [m/s]
        :param rh: (float array) relative humidity percentage [%]
        returns normal effective temperature [K]
    Reference: Li and Chan (2006)
    https://doi.org/10.1017/S1350482700001602
    """
    t2_k = kelvin_to_celsius(t2_k)
    v = scale_windspeed(va, 1.2)  # formula requires wind speed at 1.2m
    ditermeq = 1 / (1.76 + 1.4 * v**0.75)
    net = (
        37
        - ((37 - t2_k) / (0.68 - 0.0014 * rh + ditermeq))
        - 0.29 * t2_k * (1 - 0.01 * rh)
    )
    net_k = celsius_to_kelvin(net)

    return net_k


def calculate_apparent_temperature(t2_k, va, rh):
    """
    Apparent Temperature - version without radiation
        :param t2_k: (float array) 2m temperature [K]
        :param va: (float array) wind speed at 10 meters [m/s]
        :param rh: (float array) relative humidity percentage [%]
        returns apparent temperature [K]
    Reference: Steadman (1984)
    https://doi.org/10.1175/1520-0450(1984)023%3C1674:AUSOAT%3E2.0.CO;2
    See also: http://www.bom.gov.au/info/thermal_stress/#atapproximation
    """
    t2_c = kelvin_to_celsius(t2_k)
    e = calculate_nonsaturation_vapour_pressure(t2_k, rh)
    at = t2_c + 0.33 * e - 0.7 * va - 4
    at_k = celsius_to_kelvin(at)

    return at_k


def calculate_wind_chill(t2_k, va):
    """
    Wind Chill
        :param t2_k: (float array) 2m Temperature [K]
        :param va: (float array) wind speed at 10 meters [m/s]
        returns wind chill [K]
        Computation is only valid for temperatures between -50°C and 5°C and wind speeds between 5km/h and 80km/h.
        For input values outside those ranges, computed results not be considered valid.
    Reference: Blazejczyk et al. (2012)
    https://doi.org/10.1007/s00484-011-0453-2
    See also: https://web.archive.org/web/20130627223738/http://climate.weatheroffice.gc.ca/prods_servs/normals_documentation_e.html  # noqa
    """
    t2_c = kelvin_to_celsius(t2_k)  # kelvin_to_celsius(tk)
    v = va * 3.6  # convert to kilometers per hour
    windchill = 13.12 + 0.6215 * t2_c - 11.37 * v**0.16 + 0.3965 * t2_c * v**0.16
    windchill_k = celsius_to_kelvin(windchill)

    return windchill_k


def calculate_heat_index_simplified(t2_k, rh):
    """
    Heat Index
        :param t2m: (float array) 2m temperature [K]
        :param rh: (float array) relative humidity [%]
        returns heat index [K]
    Reference: Blazejczyk et al. (2012)
    https://doi.org/10.1007/s00484-011-0453-2
    """

    t2_c = kelvin_to_celsius(t2_k)

    hiarray = [
        8.784695,
        1.61139411,
        2.338549,
        0.14611605,
        1.2308094e-2,
        1.6424828e-2,
        2.211732e-3,
        7.2546e-4,
        3.582e-6,
    ]
    hi = np.copy(t2_c)

    hi_filter1 = np.where(t2_c > 20)

    hi[hi_filter1] = (
        -hiarray[0]
        + hiarray[1] * t2_c[hi_filter1]
        + hiarray[2] * rh[hi_filter1]
        - hiarray[3] * t2_c[hi_filter1] * rh[hi_filter1]
        - hiarray[4] * t2_c[hi_filter1] ** 2
        - hiarray[5] * rh[hi_filter1] ** 2
        + hiarray[6] * t2_c[hi_filter1] ** 2 * rh[hi_filter1]
        + hiarray[7] * t2_c[hi_filter1] * rh[hi_filter1] ** 2
        - hiarray[8] * t2_c[hi_filter1] ** 2 * rh[hi_filter1] ** 2
    )

    hi_k = celsius_to_kelvin(hi)

    return hi_k


def calculate_heat_index_adjusted(t2_k, td_k):
    """
    Heat Index adjusted
      :param t2_k: (float array) 2m temperature [K]
      :param td_k: (float array) 2m dewpoint temperature  [K]
      returns heat index [K]
    Reference: https://www.wpc.ncep.noaa.gov/html/heatindex_equation.shtml
    """

    rh = calculate_relative_humidity_percent(t2_k, td_k)
    t2_f = kelvin_to_fahrenheit(t2_k)

    # Simple formula
    A = 0.5 * (t2_f + 61.0 + ((t2_f - 68.0) * 1.2) + (rh * 0.094))

    # Use regression only when the average between A and air temperature is or exceeds 80°F
    warm_mask = ((A + t2_f) / 2) >= 80

    hi = A  # (T+A)/2 < 80F (reverse of warm_mask)

    # Regression formula
    rh2 = rh**2
    t2f2 = t2_f**2
    B = (
        -42.379
        + 2.04901523 * t2_f
        + 10.14333127 * rh
        - 0.22475541 * t2_f * rh
        - 6.83783e-3 * t2f2
        - 5.481717e-2 * rh2
        + 1.22874e-3 * t2f2 * rh
        + 8.5282e-4 * t2_f * rh2
        - 1.99e-6 * t2f2 * rh2
    )

    # Replace with regression where applicable
    hi[warm_mask] = B[warm_mask]

    # Adjustments — only for warm_mask points
    # 1) low humidity and high temperatures
    mask = warm_mask & (t2_f >= 80.0) & (t2_f <= 112.0) & (rh < 13.0)
    lowH_highT = ((13.0 - rh) / 4.0) * np.sqrt(
        np.maximum((17.0 - np.abs(t2_f - 95.0)), 0.0) / 17.0
    )
    hi[mask] -= lowH_highT[mask]

    # 2) High humidity and moderate temperatures
    mask = warm_mask & (t2_f >= 80.0) & (t2_f <= 87.0) & (rh > 85.0)
    # highH_modT = ((rh - 85.0) / 10.0) * ((87.0 - t2_f) / 5.0)
    highH_modT = 0.02 * (rh - 85.0) * (87.0 - t2_f)
    hi[mask] += highH_modT[mask]

    # Cold case (t ≤ 40°F)
    cold_mask = t2_f <= 40.0
    hi[cold_mask] = t2_f[cold_mask]

    # Convert to Kelvin
    hi_k = fahrenheit_to_kelvin(hi)

    return hi_k



# (C) Copyright 1996- ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.

import sys

import eccodes
from grib import decode_grib

from thermofeel.thermofeel import calculate_cos_solar_zenith_angle_integrated


# this is specific to ECMWF's forecast
def forecast_step_interval(step):
    assert step % 3 == 0  # steps are multiples of 3
    if step == 0:
        return (0, 0)
    if step <= 144:
        return (step - 3, step)
    if step > 144:
        return (step - 6, step)


def main():
    msgs = decode_grib(sys.argv[1], True)

    output = open(sys.argv[2], "wb")

    for m in msgs:
        lats = m["lats"]
        lons = m["lons"]
        assert lats.size == lons.size

        dt = m["base_datetime"]

        ftime = int(m["time"] / 100)

        step_begin, step_end = forecast_step_interval(int(m["step"]))

        print(f"Date {dt} -- Time {ftime} -- Interval [{step_begin},{step_end}]")

        cossza = calculate_cos_solar_zenith_angle_integrated(
            lat=lats,
            lon=lons,
            y=dt.year,
            m=dt.month,
            d=dt.day,
            h=ftime,
            tbegin=step_begin,
            tend=step_end,
        )

        # encode results in GRIB
        handle = eccodes.codes_clone(m["grib"])
        eccodes.codes_set_values(handle, cossza)
        eccodes.codes_write(handle, output)
        eccodes.codes_release(handle)


if __name__ == "__main__":
    sys.exit(main())



# (C) Copyright 1996- ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.


# To run this code you need to use pip install thermofeel
# import statements
import cartopy.crs as ccrs
import matplotlib.pyplot as plt
import numpy as np
from netCDF4 import Dataset  # , date2num, num2date

from thermofeel import (
    calculate_apparent_temperature,
    calculate_bgt,
    calculate_cos_solar_zenith_angle,
    calculate_cos_solar_zenith_angle_integrated,
    calculate_heat_index_adjusted,
    calculate_heat_index_simplified,
    calculate_humidex,
    calculate_mean_radiant_temperature,
    calculate_mrt_from_bgt,
    calculate_net_effective_temperature,
    calculate_relative_humidity_percent,
    calculate_saturation_vapour_pressure,
    calculate_utci,
    calculate_wbgt,
    calculate_wbgts,
    calculate_wbt,
    calculate_wind_chill,
    kelvin_to_celsius,
)
from thermofeel.thermofeel import approximate_dsrp

# read in two netcdf files containing all the variables
# to calculate the thermal indexes
my_example_nc_file2 = "radiation.nc"
fh2 = Dataset(my_example_nc_file2, mode="r")
lons = fh2.variables["longitude"][:]
lats = fh2.variables["latitude"][:]
ssrd = fh2.variables["ssrd"][0]
ssr = fh2.variables["ssr"][0]
fdir = fh2.variables["fdir"][0]
strd = fh2.variables["strd"][0]
strr = fh2.variables["str"][0]

lon_mg, lat_mg = np.meshgrid(lons, lats)

my_example_nc_file3 = "utcicomponents1.nc"
fh3 = Dataset(my_example_nc_file3, mode="r")
windspeed = np.sqrt(fh3.variables["u10"][0] ** 2 + fh3.variables["v10"][0] ** 2)
t2m = fh3.variables["t2m"][0]
td = fh3.variables["d2m"][0]

# calculate all indexes from thermofeel
rhp = calculate_relative_humidity_percent(t2k=t2m, tdk=td)
svp = calculate_saturation_vapour_pressure(tk=t2m)
cosszainstant = calculate_cos_solar_zenith_angle(
    h=6, lat=lat_mg, lon=lon_mg, y=2020, m=6, d=8
)
cosszaintegrated = calculate_cos_solar_zenith_angle_integrated(
    lat=lat_mg, lon=lon_mg, y=2020, m=6, d=8, h=6, tbegin=0, tend=6
)

approx_dsrp = approximate_dsrp(fdir=fdir, cossza=cosszainstant)

mrtinstant = calculate_mean_radiant_temperature(
    ssrd=ssrd,
    ssr=ssr,
    dsrp=approx_dsrp,
    fdir=fdir,
    strd=strd,
    strr=strr,
    cossza=cosszainstant,
)
mrtintegrate = calculate_mean_radiant_temperature(
    ssrd=ssrd, ssr=ssr, fdir=fdir, strd=strd, strr=strr, cossza=cosszaintegrated
)
utci = calculate_utci(t2_k=t2m, va_ms=windspeed, mrt_k=mrtintegrate, td_k=td)
wbgts = calculate_wbgts(t2m=t2m)
wbt = calculate_wbt(tc=kelvin_to_celsius(t2m), rh=rhp)
bgt = calculate_bgt(t_k=t2m, mrt=mrtintegrate, va=windspeed)
wbgt = calculate_wbgt(t_k=t2m, mrt=mrtintegrate, va=windspeed, td=td)
mrtbg = calculate_mrt_from_bgt(t2m=t2m, bgt=bgt, va=windspeed)
humidex = calculate_humidex(t2m=t2m, td=td)
net = calculate_net_effective_temperature(t2m=t2m, va=windspeed, td=td)
aptmp = calculate_apparent_temperature(t2m=t2m, va=windspeed)
windchill = calculate_wind_chill(t2m=t2m, va=windspeed)
hisimple = calculate_heat_index_simplified(t2m=t2m)
hia = calculate_heat_index_adjusted(t2m=t2m, td=td)

# to plot a single figure the example is for humidex index
fig = plt.figure()
ax = plt.axes(projection=ccrs.PlateCarree())
ax.coastlines()
filled_c = plt.pcolormesh(
    lon_mg, lat_mg, (humidex), transform=ccrs.PlateCarree(), cmap="RdBu_r"
)
fig.colorbar(filled_c, orientation="horizontal")
plt.show()



# (C) Copyright 1996- ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.

import unittest

import numpy as np
import pytest

import thermofeel as tmf


class TestThermalCalculator(unittest.TestCase):
    def test_relative_humidity_percent(self):
        t2_k = np.array([tmf.celsius_to_kelvin(30.0)])
        td_k = np.array([tmf.celsius_to_kelvin(28.0)])
        rhpc = np.array([tmf.calculate_relative_humidity_percent(t2_k, td_k)])
        assert rhpc == pytest.approx(89.08526710467393, abs=1e-6)

    def test_saturation_vapour_pressure(self):
        t2_k = np.array([tmf.celsius_to_kelvin(25.0)])
        svp = np.array([tmf.calculate_saturation_vapour_pressure(t2_k)])
        assert svp == pytest.approx(31.699201897293, abs=1e-6)

    def test_saturation_vapour_pressure_multiphase(self):
        t2_k = np.array([tmf.celsius_to_kelvin(-25.0)])
        phase = np.array([1])
        es = np.array(
            [tmf.calculate_saturation_vapour_pressure_multiphase(t2_k, phase)]
        )
        # assert es == pytest.approx(0.63555512, abs=1e-6) # old formula
        assert es == pytest.approx(0.63142553, abs=1e-6)

    def test_nonsaturation_vapour_pressure(self):
        t2_k = np.array([300])
        rh = np.array([87])
        svp = np.array([tmf.calculate_nonsaturation_vapour_pressure(t2_k, rh)])
        assert svp == pytest.approx(30.649976725404283, abs=1e-6)

    def test_scale_windspeed(self):
        va = np.array([7.0])
        h = np.array([2.0])
        vh = np.array([tmf.scale_windspeed(va, h)])
        assert vh == pytest.approx(5.369069989882623, abs=1e-6)

    def test_dew_point_from_relative_humidity(self):
        rh = np.array([56])
        t2_k = np.array([304.15])
        td_k = np.array([tmf.calculate_dew_point_from_relative_humidity(rh, t2_k)])
        assert td_k == pytest.approx(294.3484414118635, abs=1e-6)

    def test_mean_radiant_temperature(self):
        ssrd = np.array([60000])
        ssr = np.array([471818])
        fdir = np.array([374150])
        strd = np.array([1061213])
        strr = np.array([-182697])
        cossza = np.array([0.4])
        dsrp = np.array([tmf.approximate_dsrp(fdir, cossza)])
        mrt = np.array(
            [
                tmf.calculate_mean_radiant_temperature(
                    ssrd=ssrd / 3600,
                    ssr=ssr / 3600,
                    fdir=fdir / 3600,
                    strd=strd / 3600,
                    strr=strr / 3600,
                    cossza=cossza / 3600,
                    dsrp=dsrp / 3600,
                )
            ]
        )
        # print(f"mrt {mrt}")
        assert mrt == pytest.approx(270.85099123, abs=1e-6)

    def test_utci(self):
        # case 1
        t2_k = np.array([309.0])
        va = np.array([3])
        mrt = np.array([310.0])
        e_hPa = np.array([12])
        utci = np.array([tmf.calculate_utci(t2_k, va, mrt, td_k=None, ehPa=e_hPa)])
        assert utci == pytest.approx(307.76473586, abs=1e-5)
        # case 2
        t2_k = np.array([tmf.celsius_to_kelvin(27.0)])
        va = np.array([4])
        mrt = np.array([tmf.celsius_to_kelvin(9.2)])
        e_hPa = np.array([16.5])
        utci = np.array([tmf.calculate_utci(t2_k, va, mrt, td_k=None, ehPa=e_hPa)])
        assert tmf.kelvin_to_celsius(utci) == pytest.approx(18.93148565062157, abs=1e-5)

    def test_wbgt_simple(self):
        t2_k = np.array([tmf.celsius_to_kelvin(30.0)])
        rh = np.array([80])
        wbgts = np.array([tmf.calculate_wbgt_simple(t2_k, rh)])
        # print(f"wbgts {wbgts}")
        assert wbgts == pytest.approx(307.39508355517813, abs=1e-6)

    def test_wbt(self):
        t2_c = np.array([tmf.celsius_to_kelvin(20.0)])
        rh = np.array([50])
        wbt = np.array([tmf.calculate_wbt(t2_c, rh)])
        # print(f"wbt {wbt}")
        assert wbt == pytest.approx(286.84934189999996, abs=1e-6)

    def test_bgt(self):
        t2_k = np.array([278.15, 300, 300])
        va = np.array([20, 20, -10])  # negative va values are treated as 0
        mrt = np.array([278.15, 310, 310])
        bgt = np.array([tmf.calculate_bgt(t2_k, va, mrt)])
        # print(f"bgt {bgt}")
        assert bgt[0, 0] == pytest.approx(277.1238737724192, abs=1e-6)
        assert bgt[0, 1] == pytest.approx(298.70218703427656, abs=1e-6)
        assert bgt[0, 2] == pytest.approx(298.70216299754475, abs=1e-6)

    def test_wbgt(self):
        t2_k = np.array([300])
        td_k = np.array([290])
        va = np.array([20])
        mrt = np.array([310])
        wbgt = np.array([tmf.calculate_wbgt(t2_k, mrt, va, td_k)])
        # print(f"wbgt {wbgt}")
        assert wbgt[0] == pytest.approx(295.5769818634555, abs=1e-6)
        # # test negative values are treated as 0
        # va[0] = -10
        # wbgt = np.array([tmf.calculate_wbgt(t_k, mrt, va, td_k)])
        # # print(f"wbgt {wbgt}")
        # assert wbgt[0] == pytest.approx(295.5769818634555, abs=1e-6)

    def test_mrt_from_bgt(self):
        t2_k = np.array([tmf.celsius_to_kelvin(25.0)])
        bgt_k = np.array([tmf.celsius_to_kelvin(23.0)])
        # print(f"bgt_k {bgt_k}")
        va = np.array([10])
        mrt_c = np.array([tmf.calculate_mrt_from_bgt(t2_k, bgt_k, va)])
        assert mrt_c == pytest.approx(279.80189775556704, abs=1e-6)

    def test_humidex(self):
        t2_k = np.array([304])
        td_k = np.array([300])
        hu = np.array([tmf.calculate_humidex(t2_k, td_k)])
        # print(f"hu {hu}")
        assert hu == pytest.approx(318.4601286141123, abs=1e-6)

    def test_normal_effective_temperature(self):
        t2_k = np.array([307])
        va = np.array([4])
        rh = np.array([80])
        net = np.array([tmf.calculate_normal_effective_temperature(t2_k, va, rh)])
        # print(f"net {net}")
        assert net == pytest.approx(304.13650125, abs=1e-6)

    def test_apparent_temperature(self):
        t2_k = np.array([tmf.celsius_to_kelvin(25.0)])
        va = np.array([3])
        rh = np.array([75])
        at = np.array([tmf.calculate_apparent_temperature(t2_k, va, rh)])
        # print(f"at {at}")
        assert at == pytest.approx(299.86678322384626, abs=1e-6)

    def test_wind_chill(self):
        t2_k = np.array([270])
        va = np.array([10])
        wc_k = np.array([tmf.calculate_wind_chill(t2_k, va)])
        assert wc_k == pytest.approx(261.92338925380074, abs=1e-6)
        # reference result from from wikipedia article https://en.wikipedia.org/wiki/Wind_chill
        t2_k = np.array([tmf.celsius_to_kelvin(-20)])  # -20C to K
        va = np.array([5 / 3.6])  # 5 km/h to m/s
        wc_k = np.array([tmf.calculate_wind_chill(t2_k, va)])
        wc_c = tmf.kelvin_to_celsius(wc_k)
        assert wc_c == pytest.approx(
            -24.27850328, abs=1e-6
        )  # around ~ -24C for wind chill (not exact)
        va = np.array([30 / 3.6])  # 30 km/h to m/s
        wc_k = np.array([tmf.calculate_wind_chill(t2_k, va)])
        wc_c = tmf.kelvin_to_celsius(wc_k)
        print(f"wc_c {wc_c}")
        assert wc_c == pytest.approx(
            -32.56804448, abs=1e-6
        )  # around ~ -33C for wind chill (not exact)

    def test_heat_index_simplified(self):
        t2_k = np.array([tmf.celsius_to_kelvin(21.0)])
        rh = np.array([80])
        hi = np.array([tmf.calculate_heat_index_simplified(t2_k, rh)])
        # print(f"hi {hi}")
        assert hi == pytest.approx(294.68866082, abs=1e-6)

    def test_heat_index_adjusted(self):
        t2_k = np.array([295])
        td_k = np.array([290])
        hia = np.array([tmf.calculate_heat_index_adjusted(t2_k, td_k)])
        # print(f"hia {hia}")
        assert hia[0] == pytest.approx(295.15355699, abs=1e-6)


if __name__ == "__main__":
    unittest.main()  # pragma: no cover



# (C) Copyright 1996- ECMWF.
#
# This software is licensed under the terms of the Apache Licence Version 2.0
# which can be obtained at http://www.apache.org/licenses/LICENSE-2.0.
# In applying this licence, ECMWF does not waive the privileges and immunities
# granted to it by virtue of its status as an intergovernmental organisation
# nor does it submit to any jurisdiction.

import sys

from Magics import macro as magics


def main():
    data = magics.mgrib(
        grib_input_file_name=sys.argv[1],
    )

    projection = magics.mmap(
        subpage_map_library_area="on",
        subpage_map_area_name="global",
        page_id_line="off",
    )

    coast = magics.mcoast()

    legend = magics.mlegend(legend_display_type="continuous")

    # contour = magics.mcont(
    #     contour_automatic_setting = "ecmwf",
    #     legend = "on")

    output = magics.output(
        output_formats=["png"],
        output_width=1200,
        output_cairo_transparent_background="on",
        output_name_first_page_number="off",
        output_name="magics",
    )

    contour = magics.mcont(
        contour="off",
        contour_level_selection_type="interval",
        contour_shade="on",
        contour_shade_method="area_fill",
        contour_interval=1.0,
        contour_shade_min_level=-50.0,
        contour_shade_max_level=50.0,
        contour_interpolation_ceiling=49.99,
        contour_interpolation_floor=-49.99,
        contour_label="off",
        contour_shade_colour_method="list",
        contour_shade_colour_list=[
            "#053061",
            "#073466",
            "#09376b",
            "#0b3b70",
            "#0e3f75",
            "#10427a",
            "#12467e",
            "#154a83",
            "#174e88",
            "#1a528c",
            "#1c5691",
            "#1f5a95",
            "#225e99",
            "#24629e",
            "#2766a2",
            "#2a6aa6",
            "#2c6faa",
            "#2f73ad",
            "#3277b1",
            "#357cb4",
            "#3880b8",
            "#3a85bb",
            "#3d89be",
            "#408ec0",
            "#4392c3",
            "#4a97c5",
            "#529bc7",
            "#5a9fc9",
            "#61a3cc",
            "#68a7ce",
            "#70abd0",
            "#77afd2",
            "#7eb3d4",
            "#85b7d7",
            "#8cbbd9",
            "#93bfdb",
            "#9ac3dd",
            "#a1c7df",
            "#a8cbe1",
            "#afcfe3",
            "#b6d3e5",
            "#bdd7e7",
            "#c4dbe9",
            "#cbdfeb",
            "#d2e3ed",
            "#dae7ef",
            "#e1ebf1",
            "#e8eff3",
            "#f0f3f5",
            "#f7f7f7",
            "#f7f7f7",
            "#f8f1ef",
            "#f8ebe6",
            "#f9e6de",
            "#f9e0d6",
            "#f9dace",
            "#f9d5c7",
            "#f8cfbf",
            "#f8c9b8",
            "#f7c4b1",
            "#f6beaa",
            "#f5b8a3",
            "#f4b39c",
            "#f3ad95",
            "#f1a78f",
            "#f0a289",
            "#ee9c83",
            "#ec977d",
            "#ea9177",
            "#e88b71",
            "#e6866c",
            "#e48067",
            "#e27b62",
            "#df755d",
            "#dd6f58",
            "#da6954",
            "#d86450",
            "#d55e4c",
            "#d15848",
            "#ce5345",
            "#ca4e42",
            "#c64940",
            "#c2443d",
            "#bd3f3a",
            "#b93a38",
            "#b43536",
            "#af3033",
            "#ab2c31",
            "#a5272f",
            "#a0232d",
            "#9b1e2c",
            "#961a2a",
            "#901628",
            "#8a1227",
            "#850e25",
            "#7f0a24",
            "#790623",
            "#730321",
            "#6d0120",
            "#67001f",
        ],
        legend="on",
    )

    contour = magics.mcont(
        contour_level_selection_type="count",
        legend="on",
        contour_level_tolerance=0,
        contour_level_count=11,
        contour_shade="on",
        contour_shade_method="area_fill",
        contour_shade_colour_method="palette",
        contour_shade_palette_name="colorbrewer_ext_YlGn_11_r",
    )

    contour = magics.mcont(
        contour_level_selection_type="count",
        legend="on",
        contour_level_tolerance=0,
        contour_level_count=11,
        contour_shade="on",
        contour_shade_method="area_fill",
        contour_shade_colour_method="palette",
        contour_shade_palette_name="m_green_purple_11",
    )

    # contour = magics.mcont(
    # contour = 'off',
    # contour_hilo = 'off',
    # # contour_interval = 2.0,
    # contour_label = 'off',
    # # contour_level_selection_type = 'interval',
    # contour_shade = 'on',
    # contour_shade_colour_method = 'calculate',
    # # contour_shade_max_level = 22.0,
    # contour_shade_method = 'area_fill',
    # # contour_shade_min_level = -28.0,
    # legend = 'on'
    # )

    magics.plot(output, projection, data, contour, coast, legend)


if __name__ == "__main__":
    sys.exit(main())
